/* FILE: metcheck.c
 *
 * PROGRAM: Metadata, Descriptor check (metcheck). 
 *
 * DESCRIPTION: This file contains the main function for the Metadata - Descriptor tool.
 *     
 *
 * RCS INFO:
 *
 *      $Id: 
 *
 *      $Log: 
 *
 *
 *   06/20/00  Amer Rahabi  Original           
*/
/* INCLUDED FILES:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "metcheck.h"
#include "checkI.h"
#include "odldef.h"
#include "odlinter.h"
#include "lstList.h"
#include "odlFieldList.h"
/* ---------------------------------------------------------------------*/

/* GLOBAL VARIABLES: the following variables may be used in other files.*/

metArgStructure CONTROLS; /* metcheck controls */

/* ---------------------------------------------------------------------*/
/* STATIC VARIABLES: the following variables are for the exclusive use
 *     of the functions defined within this file.
*/
/* Exit status to return to the operating system. */
const int ExitSuccess =  0;
const int ExitAbort = -1;

/* Return status to return to a calling function. */
const int Success = 1;
const int Failure = 0;
/* ---------------------------------------------------------------------*/

/* STATIC PROTOTYPES: the following prototypes are for functions defined
 *     and used exclusively within this file.
*/
static int CmdLineProc (int argc, char **argv, metArgStructure *CONTROLS);

/* ---------------------------------------------------------------------*/

/*****************************************************************************
 * 
 * NAME: main
 *
 * DESCRIPTION: This is the main function of the metcheck tool.
 *
 * PARAMETERS:
 *     argc (int)              The number of command line arguments.
 *     argv (char **)          An array of character pointers.  Each pointer
 *                             points to one of the command line arguments.
 *
 * RETURN VALUE:
 *     Exit status = ExitSuccess --> The metcheck tool completed successfully,
 *                                   and created an out put file contains all discrpencies.
 *                                   and warnings.
 *                 = ExitAbort --> The metcheck tool encountered a fatal error
 *                                   and could not complete successfully.
 *     
 * GLOBAL VARIABLES:
 *     CONTROLS (metArgStructure)  A data structure containing various controls
 *                                of the metcheck tool.
 *	
 * PRE- & POST-CONDITIONS:
 *     None.
 *
 * NOTES:
 *     None.
 *
 * REVISION HISTORY:
 *
 * 6/20/2000  Amer Rahabi  Original 
 *
 ****************************************************************************/

main(int argc, char **argv)
{
	
	AGGREGATE  pRootMET, pRootDSC; /*  pointers to the root nodes */
	
 /* AGGREGATE mainAgg, tmpAgg;
  lstLIST fldList;
  descList fldEntry; */
	  
  /* Process the command line options, and initialize the metcheck controls: */
    if( CmdLineProc(argc, argv, &CONTROLS) != Success ) exit( ExitAbort );

	/* Initialize the utility modules: */


  /* Create the root node of the met file  tree: */
	if (  ( pRootMET = NewAggregate(NULL, KA_GROUP, "MET_ROOT", NULL) ) == NULL  )
		checkFatalError ( "metcheckMain",
                   "Creation of the met file lable's root node failed" );
/* check the met label for syntax errors and out put the errors if any: */

  if (CheckSynErr(CONTROLS.pMetFile, pRootMET) != Success )
      exit( ExitAbort );
  
 /* Create the root node of the Descriptor file : */


	if (  ( pRootDSC = NewAggregate (NULL, KA_GROUP, "DSC_ROOT", NULL) ) == NULL  )
		checkFatalError( "metcheckMain",
                   "Creation of the Descriptor file's root node failed" ); 

/* check the Dscriptor label for syntax errors and out put the errors if any: */
	
   if (CheckSynErr(CONTROLS.pDscFile, pRootDSC) != Success )
     checkFatalError( "CheckSynErr",
                   "Syntax error were found and the program has to Abort" );  
     
     /*exit( ExitAbort ); */
  
   
 /*  The following process will build lstLIST for the descrioptor file. Also */
 /*  will check the path for each aggragate then will check if all Mandatory Aggragates exist */
 /*  the metdata file .  */
 
   if (ManObjCheck( pRootMET, pRootDSC, CONTROLS.pOutputFile) != Success)
     checkWarnings( "ManObjCheck",
                   "Warning  :: some errors were found while checking " );
 
 /* The following process will do the checking process for all normal aggregates and */
 /* verifies all their values and and their types  */

   if (CheckNormal( pRootMET, pRootDSC, CONTROLS.pOutputFile) != Success)
     checkWarnings( "CheckNormal",
                   "Warning  :: some errors were found while checking " );
     

 /* test the odl with test.c */


 /* test(pRootMET, CONTROLS.pOutputFile); */

/**/


 	exit( ExitSuccess );
}

/*****************************************************************************
 * 
 * NAME: CmdLineProc
 *
 * DESCRIPTION: This function processes the command line options.  It verifies
 *     that the options are valid for the metcheck tool.  It also initializes the
 *     metcheck control structure to default values, which may be overridden via
 *     command line options.
 *
 * PARAMETERS:
 *     argc (int)              The number of command line arguments.
 *     argv (char **)          An array of character pointers.  Each pointer
 *                             points to one of the command line arguments.
 *     controls (metArgStructure)  The metcheck's control structure.  It is initialized
 *                             by this function.
 *
 * RETURN VALUE:
 *     CmdLineProc (int)  Success --> This function completed successfully.
 *                        Failure --> This function encountered an error and
 *                                    could not complete successfully.
 *     
 * GLOBAL VARIABLES:
 *     None.
 *	
 * PRE- & POST-CONDITIONS:
 * 
 *     1. PRE:  An invalid option given on the command line.
 *        POST: Error message generated stating such.  The function returns a
 *              status of "Failed".
 *     2. PRE:  An option's argument given that begins with a dash--illegal
 *              syntax.
 *        POST: Error message generated stating such.  The function returns a
 *              status of "Failed".
 *     5. PRE:  An option with an argument that is a file name.  The file does
 *              not exist.
 *        POST: Error message generated stating such.  The function returns a
 *              status of "Failed".
 *
 * NOTES:
 *
 *     The name of an option's argument can not begin with a dash ("-").  If it does, then
 *     this function considers this an error: a message is generated to
 *     STDERR or the debgu file, and this function returns with a status of
 *     "failed".
 *
 *
 * REVISION HISTORY:
 *     07/17/00  Amer Rahabi
 *
 ****************************************************************************/

static int CmdLineProc (int argc, char **argv, metArgStructure *controls)
{
  /* The following three extern variables are from getopt. */
  extern char *optarg;
  extern int  optind;
  extern int  opterr;
 /* const char  *metfiles = "example.met"; */
  const char *ValidOptions     = "dvf:g:m:o:";
  const char Dash              = '-';
  /* */

  enum {None, NoArg, InvalidOption, InvalidOptionArg, OpenFailureDSC, OpenFailureMET,
        OpenFailureOutput, OpenFailureDebug, MallocFailureComment,
        MallocFailureTitle} error;

  int opt;

  opterr = 0;   /* disable error messages in getopt */
  error = None;

/* Initialize the fields of the metcheck's controls structure to their default
   values.*/ 

  controls->pDscFile    = NULL;
  controls->pMetFile    = NULL;
 /* */
/*   */
  controls->pOutputFile  = stdout;
  controls->verbose    = controls->debug = 0;
  controls->pDebugFile = stderr; 

/* Retrieve and process each option given on the command line.  This continues
   until all options are exhauseted or an error is encountered. */

 
  while ((error == None) && ((opt = getopt (argc, argv, ValidOptions)) != EOF))
  { 

    switch ((char) opt)
    {

      /* Debug option (d).  It has no arguments. */

      case 'd':
        controls->debug = 1;
        break;

      /* Input desc file option (f).  Its argument is the name of a file .desc */

      case 'f':
        if (optarg[0] == Dash)
          error = InvalidOptionArg;
        else
          if ((controls->pDscFile = fopen (optarg, "r")) == NULL)
            error = OpenFailureDSC;
        break;

      /* Error message file option (g).  Its argument is the name of a file
         to which any error or debug messages are written. */

      case 'g':
        if (optarg[0] == Dash)
          error = InvalidOptionArg;
        else
			    if ((controls->pDebugFile = fopen (optarg, "w")) == NULL)
                error = OpenFailureDebug;
          		
        break; 

      /* Meta data file option (m).  Its argument is the name of a
         .met file. */

      case 'm':
        if (optarg[0] == Dash)
          error = InvalidOptionArg;
        
        else         
          if ((controls->pMetFile = fopen (optarg, "r")) == NULL)
            error = OpenFailureMET; 
          /*  printf(" the metdata file is %s\n ",controls.pMetFile); */
          /*  printf("this is the value of optarg = %s\0", optarg); */
        break;

      /* Output file option (o).  It argument is the name of a file to which
         the output report is written. */

      case 'o':
        if (optarg[0] == Dash)
          error = InvalidOptionArg;
        else
          if ((controls->pOutputFile = fopen (optarg, "w")) == NULL)
            error = OpenFailureOutput;
        break;

       /* Verbose option (v).  It has no arguments. */

      case 'v':
        controls->verbose = 1;
        break;

      /* Invalid option or missing mandatory argument of an option. */

      case '?':
        error = InvalidOption;
        break;
    }
  }
  
 
  /* Check to see if we have enough arguments in the command line */

  if(   (opt == EOF) && ( controls->pDscFile == NULL ) &&
              (controls->pMetFile == NULL)   )
        error = NoArg;

  /* If no errors were encountered processing the command line options, then
     this function has completed successfully. */
  if (error == None)
    return (Success);

  /* Otherwise, an error has occurred when processing the command line
     options.  Output the appropriate error message to standard error, and
     display the correct usage for the metcheck tool. */

  else
  {
    switch (error)
    {
      
      case NoArg:
        fprintf (controls->pDebugFile, 
			     "There are no sufficient arguments entered.\n"
           "At least -f <desc_file> and -m <met_file> should be entered \n ");
        break;                         

      case InvalidOptionArg:
        fprintf (controls->pDebugFile, 
			     "Option argument's name can not begin with a "
                         "dash ('-')\n");
        break;
      case OpenFailureDSC:
        fprintf (controls->pDebugFile, "Failed to open the given desc file%s\n",controls->pDscFile);
        break;
      case OpenFailureMET:
        fprintf (controls->pDebugFile, 
			    "Failed to open the given Meta data file %s\n",controls->pMetFile );
        break;
      case OpenFailureOutput:
        fprintf (controls->pDebugFile, 
			    "Failed to open the given output file \n");
        break;
      case OpenFailureDebug:
        fprintf (controls->pDebugFile, 
			"Failed to open the given debug/error file\n");
        break;
      case InvalidOption:
        fprintf (controls->pDebugFile, 
			    "Invalid option or missing option's argument\n");
        break;
    }
    fprintf (controls->pDebugFile, "\nUsage: metcheck [-d|-v|-f <desc_file>|"
                     "-g <debug_file>|\n            -m <met_file>|"
                     "-o <output_file>\n");
    return (Failure);
  }
} /* end CmdLineProc */

/*****************************************************************************
 * 
 * NAME: checkSynErr( check Syntax errors)
 *
 * DESCRIPTION: Reports Syntax errors and abort further porcessing.
 *
 * PARAMETERS:  
 *     i_file -- (File *) pointer to a file to be checked
 *                      
 *     root         --  root label
 *                      
 *
 * RETURN VALUE: None
 *     
 * GLOBAL VARIABLES:
 *     pOutputFile -- FILE The the out put file to which error messages are to be 
 *                   written.
 *	
 * REVISION HISTORY:
 *   07/22/00  Amer Rahabi             
 *
 ****************************************************************************/


static int CheckSynErr (FILE *i_file, AGGREGATE  proot)

{
 
/* Create the root node of the specified file  tree: */
	 
/*	if ( ReadLabel (i_file, proot) != Success ) */
	
  int ret=ReadLabel (i_file, proot);
	if (ret!=1) {
	    fprintf(CONTROLS.pOutputFile,
                "Error reading ODL label\n");    
                

 
	fprintf(CONTROLS.pOutputFile,"-------- begin \"%s\" ----------\n",i_file);
 	PrintLabel(proot);
	/*fflush(stdout); */
	fprintf(CONTROLS.pOutputFile,"--------   end \"%s\" ----------\n",i_file);


/*	RemoveAggregate(proot); */

    return (Failure);
    }
  else
    return (Success);
    
}
/* end checkSynError */

/*****************************************************************************
 * 
 * NAME: checkErrReport
 *
 * DESCRIPTION: Reports All errors and wrnings to the out put file .
 *
 * PARAMETERS:  
 *     header       -- (char *) pointer to a buffer containing header information 
 *                      of errors in the same catagory.
 *     errlevel     -- (char *) pointer to string to specifie if it is an ERROR or 
 *                      a WARNING.
 *     msg          -- (char *) pointer to a buffer containing a description
 *                      of the error that occurred.
 *
 * RETURN VALUE: None
 *     
 * GLOBAL VARIABLES:
 *     pOutputFile -- FILE The out put file to which error messages are to be 
 *                   written.
 *	
 * REVISION HISTORY:
 *   06/19/00  Amer Rahabi             
 *
 ****************************************************************************/

void checkHeader( char *header)
{
  if (header != NULL) 
  {
  fprintf( CONTROLS.pOutputFile, 
  "/ ***************************************************************************/\n");
  fprintf( CONTROLS.pOutputFile, 
  "  %s                                                                          \n",header);                                                                           
  fprintf( CONTROLS.pOutputFile, 
  "/ ***************************************************************************/\n");

  }
  
} /* end checkErrReport */

/*****************************************************************************
 * 
 * NAME: checkFatalError
 *
 * DESCRIPTION: Reports fatal errors and shuts down the program.
 *
 * PARAMETERS:  
 *     functionName -- (char *) pointer to a string containing the name of
 *                      the function that trapped the fatal error.
 *     msg          -- (char *) pointer to a buffer containing a description
 *                      of the error that occurred.
 *
 * RETURN VALUE: None
 *     
 * GLOBAL VARIABLES:
 *     pDebugFile -- FILE The debug file to which Fatal errors messages are to be 
 *                   written.
 *	
 * REVISION HISTORY:
 *   06/19/00  Amer Rahabi             
 *
 ****************************************************************************/

void checkFatalError( char *functionName, char *msg )
{
  fprintf( CONTROLS.pDebugFile, "%s: FATAL - %s\n", functionName, msg );
	exit(ExitAbort);

} /* end checkFatalError */

/*****************************************************************************
 * 
 * NAME: checkWarnings
 *
 * DESCRIPTION: Reports warnings if the verbose flag in the metcheck control
 *     structure is set (= 1).
 *
 * PARAMETERS:  
 *     functionName -- (char *) pointer to a string containing the name of
 *                      the function that trapped the warnings.
 *     msg          -- (char *) pointer to a buffer containing a description
 *                      of the error that occurred.
 *
 * RETURN VALUE: None
 *     
 * GLOBAL VARIABLES:
 *     CONTROLS.verbose  -- (int)  Flag indicating whether the program is
 *                                    in verbose mode (= 1) or not (= 0).
 *     CONTROLS.pDebugFile
 *                       -- (FILE) The debug file to which error, and warnings 
 *                                    messages are to be written.
 *	
 * REVISION HISTORY:
 *   06/20/00  Amer Rahabi             
 *
 ****************************************************************************/

void checkWarnings( char *functionName, char *msg )
{
	if( !CONTROLS.verbose ) return;

  fprintf( CONTROLS.pDebugFile, "%s: WARNING - %s\n", functionName,
                                                              msg );
	return;

} /* end checkWarnings */
